@@ -10,10 +10,14 @@
 #include "sox_i.h"
 #include "g711.h"
 
+typedef uint16_t uint14_t;
+typedef uint16_t uint13_t;
+typedef int16_t int14_t;
+typedef int16_t int13_t;
 #define SOX_ULAW_BYTE_TO_SAMPLE(d,clips)   SOX_SIGNED_16BIT_TO_SAMPLE(sox_ulaw2linear16(d),clips)
 #define SOX_ALAW_BYTE_TO_SAMPLE(d,clips)   SOX_SIGNED_16BIT_TO_SAMPLE(sox_alaw2linear16(d),clips)
-#define SOX_SAMPLE_TO_ULAW_BYTE(d,c) sox_14linear2ulaw(SOX_SAMPLE_TO_SIGNED_16BIT(d,c) >> 2)
-#define SOX_SAMPLE_TO_ALAW_BYTE(d,c) sox_13linear2alaw(SOX_SAMPLE_TO_SIGNED_16BIT(d,c) >> 3)
+#define SOX_SAMPLE_TO_ULAW_BYTE(d,c) sox_14linear2ulaw(SOX_SAMPLE_TO_UNSIGNED(14,d,c) - 0x2000)
+#define SOX_SAMPLE_TO_ALAW_BYTE(d,c) sox_13linear2alaw(SOX_SAMPLE_TO_UNSIGNED(13,d,c) - 0x1000)
 
 int lsx_rawseek(sox_format_t * ft, uint64_t offset)
 {
@@ -27,19 +31,19 @@ int lsx_rawstart(sox_format_t * ft, sox_bool default_rate,
                  sox_encoding_t encoding, unsigned size)
 {
   if (default_rate && ft->signal.rate == 0) {
-    lsx_warn("'%s': sample rate not specified; trying 8kHz", ft->filename);
+    lsx_warn("`%s': sample rate not specified; trying 8kHz", ft->filename);
     ft->signal.rate = 8000;
   }
 
   if (default_channels && ft->signal.channels == 0) {
-    lsx_warn("'%s': # channels not specified; trying mono", ft->filename);
+    lsx_warn("`%s': # channels not specified; trying mono", ft->filename);
     ft->signal.channels = 1;
   }
 
   if (encoding != SOX_ENCODING_UNKNOWN) {
     if (ft->mode == 'r' && ft->encoding.encoding != SOX_ENCODING_UNKNOWN &&
         ft->encoding.encoding != encoding)
-      lsx_report("'%s': Format options overriding file-type encoding",
+      lsx_report("`%s': Format options overriding file-type encoding",
                  ft->filename);
     else
       ft->encoding.encoding = encoding;
@@ -48,7 +52,7 @@ int lsx_rawstart(sox_format_t * ft, sox_bool default_rate,
   if (size != 0) {
     if (ft->mode == 'r' && ft->encoding.bits_per_sample != 0 &&
         ft->encoding.bits_per_sample != size)
-      lsx_report("'%s': Format options overriding file-type sample-size",
+      lsx_report("`%s': Format options overriding file-type sample-size",
                  ft->filename);
     else
       ft->encoding.bits_per_sample = size;
@@ -67,7 +71,9 @@ int lsx_rawstart(sox_format_t * ft, sox_bool default_rate,
       sox_format_t * ft, sox_sample_t *buf, size_t len) \
   { \
     size_t n, nread; \
+    SOX_SAMPLE_LOCALS; \
     ctype *data = lsx_malloc(sizeof(ctype) * len); \
+    LSX_UNUSED_VAR(sox_macro_temp_sample), LSX_UNUSED_VAR(sox_macro_temp_double); \
     nread = lsx_read_ ## type ## _buf(ft, (uctype *)data, len); \
     for (n = 0; n < nread; n++) \
       *buf++ = cast(data[n], ft->clips); \
@@ -92,8 +98,10 @@ READ_SAMPLES_FUNC(df, sizeof(double), su, double, double, SOX_FLOAT_64BIT_TO_SAM
   static size_t sox_write_ ## sign ## type ## _samples( \
       sox_format_t * ft, sox_sample_t const * buf, size_t len) \
   { \
+    SOX_SAMPLE_LOCALS; \
     size_t n, nwritten; \
     ctype *data = lsx_malloc(sizeof(ctype) * len); \
+    LSX_UNUSED_VAR(sox_macro_temp_sample), LSX_UNUSED_VAR(sox_macro_temp_double); \
     for (n = 0; n < len; n++) \
       data[n] = cast(buf[n], ft->clips); \
     nwritten = lsx_write_ ## type ## _buf(ft, (uctype *)data, len); \
